//
// File: CGameControl.h
// Created by: berny <2berny@seznam.cz>
// Created on: Mon Aug  4 09:18:20 2003
//

#include <stdio.h>
#include <assert.h>
#include <ctime>

using namespace std;

#include "CGameControl.h"
#include "CAI.h"
#include "CAIReplay.h"

CGameControl::CGameControl()
{
	m_valid = false;
	m_AI[0] = NULL;
	m_AI[1] = NULL; 
	m_deleteAI = false;
}


CGameControl::~CGameControl()
{
	// smazeme stare AI, pokud je treba
	if ( m_deleteAI ) {
		if ( m_AI[0] != NULL ) delete m_AI[0];
		if ( m_AI[1] != NULL ) delete m_AI[1];
	}
}

void CGameControl::NewGame(int cells, int rocks, CAI *pl1, CAI *pl2, bool deleteAI)
{
	assert( pl1 != NULL && pl2 != NULL && cells < MAX_CELLS && cells > 1 && rocks > 0 );
	
	// nejake drobne testy na spravnost argumentu
	if (cells >= MAX_CELLS) {
		fprintf(stderr, "Prilis velky rozmer hraci plochy\n");
		return;		
	}
	if (pl1 == NULL || pl2 == NULL) {
		fprintf(stderr, "U GAI nesmi byt parametr == NULL\n");
		return;
	}
	if (cells < 2 || rocks < 1) {
		fprintf(stderr, "Nesmyslne male rozmery pro hru nebo nedostatek kamenu\n");
		return;
	}
	
	// smazeme stare AI, pokud je treba
	if ( m_deleteAI ) {
		if ( m_AI[0] != NULL ) delete m_AI[0];
		if ( m_AI[1] != NULL ) delete m_AI[1];
	}
	
	// nastaveni pro novou hru
	m_gm_cells = cells;
	m_gm_rocks = rocks;
	while ( !m_gm_history.empty() ) m_gm_history.pop_back();
	
	// pripravime hraci plochu
	m_status.cells = cells;
	for (int i = 1; i <= cells; i++) {
		m_status.field[0][i] = rocks;
		m_status.field[1][i] = rocks;
	}
	m_status.field[0][0] = 0;
	m_status.field[1][0] = 0;
	m_status_changed = true;
	
	// priprava hracu
	m_AI[0] = pl1;
	m_AI[1] = pl2;
	m_deleteAI = deleteAI;
	m_status.player = 0;
	m_time[0] = 0;
	m_time[1] = 0;
	
	// dalsi zalezitosti
	m_rocks   = cells * 2 * rocks; // celkovy pocet kamenu ve hre
	m_turnscnt = 0;
	
	// a rekneme, ze jsme pripraveni hrat
	m_ready_next_move = true;
	
	// rekneme, ze data jsou platna
	m_valid = true;
}
	
// AI vraci vysledek se hry - testujeme, zda nas vola objekt, ktery je skutecne na tahu!!!!
void CGameControl::AIMoved(int field, const CAI *which)
{
	assert ( which != NULL && m_valid == true );
	
	// aby nahodou druhy hrac nepodvadel :-)
	if ( which != m_AI[m_status.player] ) {
		printf("Nejaky hrac se snazi podvadet a hrat i za soupere :-)\n");
		return;
	}
	
	// zda je tah "povolen"
	if ( !gsCanMove(&m_status, field) ) {
		printf("Pokus o neplatny tah.\n");
		// dame dalsi sanci
		m_ready_next_move = true;
		return;
	}
	
	// zmerime hraci cas, jak dlouho mu to trvalo
	time_t t1 = time(NULL);
	m_time[m_status.player] += difftime(t1, m_clockstart);
	
	// provedeme tedy tah
	gsMakeMove(&m_status, field);
	m_turnscnt++;
	printf("Move :%d\n", field);
	
	// ulozime ho do seznamu tahu
	m_gm_history.push_back(field);
	
	// kdyby nahodou
	if ( GameEnded() ) return;
		
	// podivame, zda muze aktualni hrac tahnou
	if ( !gsCanMove(&m_status) ) {
		if ( m_status.player == 1 ) m_status.player = 0;
		else m_status.player = 1;
	}
	
	// byl proveden tah, takze bude treba prekleslit obrazovku
	m_status_changed  = true;
	
	// a jsme pripraveni na dalsi tah
	m_ready_next_move = true;
}
	
// testuje, zda hra zkoncila, pripadne kdo vyhral
int CGameControl::GameEnded()
{
	assert( m_valid == true );
	
	// hra konci, kdyz vsechny kameny v domecku jsou
	if ( m_status.field[0][0] + m_status.field[1][0] < m_rocks ) return GAME_ENDED_NO;
		
	// Test, kdo ma vic - mohly by tam byt else, ale tohle je na pohed hezci a neni to casove kriticka cast :-)
	if ( m_status.field[0][0] >  m_status.field[1][0] ) return GAME_ENDED_1ST;
	if ( m_status.field[0][0] <  m_status.field[1][0] ) return GAME_ENDED_2ND;
	if ( m_status.field[0][0] == m_status.field[1][0] ) return GAME_ENDED;
		
	return GAME_ENDED_NO; // aby to pri kompilaci nepsalo warning
}
	
// vraci true, pokud doslo ke zmene hraci plochy od posledniho volani - pouziva se pri prekleslovani
bool CGameControl::StatusChanged()
{
	if ( m_status_changed ) {
		m_status_changed = false;
		return true;
	}
	else return false;
}
	
// vraci status hry
st_game_status CGameControl::GetStatus()
{
	assert( m_valid == true );

	return m_status;	
}

// pripadne provede dalsi tah
void CGameControl::NextTurn()
{
	assert( m_valid == true );

	// pokud se ceka na dalsi tah, tak ho provedeme
	if ( m_ready_next_move ) {
		m_ready_next_move = false;
		m_AI[m_status.player]->TryMove();
		m_clockstart = time(NULL);
	}
}

//	vraci, kolik casu ktery hrac uz potreboval na svou hru
double CGameControl::GetTime(int player)
{
	assert( (player == 0 || player == 1) && m_valid == true );
	return m_time[player];
}		


// vraci cislo tahu
int CGameControl::GetTurnsCnt()
{
	assert( m_valid == true );

	return m_turnscnt;
}

// zavola TimerTick u inteligenci
void CGameControl::TimerTick()
{
	assert( m_valid == true );
	
	if ( m_AI[0] != NULL) m_AI[0]->TimerTick();
	if ( m_AI[1] != NULL) m_AI[1]->TimerTick();
}

// ulozi hru - ukladat lze jen dohrane hry
bool CGameControl::SaveHistory(char *filename)
{
	assert( m_valid == true );

	if ( !gsGameEnded(&m_status) || filename == NULL ) return false;
	
	// otevreme soubor
	FILE *f = fopen(filename, "wt");
	if ( f == NULL ) return false;

	// zapiseme hlavicku
	fprintf(f, "%d\n%d\n%d\n", m_gm_cells, m_gm_rocks, m_gm_history.size());
	
	// zapiseme tahy hracu
	for ( unsigned int i = 0; i < m_gm_history.size(); i++)
		fprintf(f, "%d\n", m_gm_history[i]);
	
	// konec
	fclose(f);	
	
	return true;
}
	
// nacte hru a pripravi si inteligence, ktere hraji "replay"
bool CGameControl::LoadHistory(char *filename)
{
	if ( filename == NULL ) return false;
	
	// otevreme soubor
	FILE *f = fopen(filename, "rt");
	if ( f == NULL ) return false;

	int tmp, tmp2; 
	
	// nacteme hlavicku
	fscanf(f, "%d\n%d\n%d\n", &m_gm_cells, &m_gm_rocks, &tmp);
	
	m_gm_loaded.reserve(tmp); // zerezvujeme misto v seznamu tahu - neni to nutne, ale je to takove pekne elegantni :-)
	
	// zapiseme tahy hracu
	for ( int i = 0; i < tmp; i++) {
		fscanf(f, "%d\n", &tmp2);
		m_gm_loaded.push_back(tmp2);
	}
	
	// zavreme soubor
	fclose(f);	
	
	// zahajime hru
	CAIReplay *ai1, *ai2;
	
	ai1 = new CAIReplay(this);
	ai2 = new CAIReplay(this);
	
	NewGame(m_gm_cells, m_gm_rocks, ai1, ai2, true);
	
	return true;	
}

// zda data jsou platna - vraci m_valid
bool CGameControl::IsValid()
{
	return m_valid;
}
