//
// File: Global.cc
// Created by: berny <2berny@seznam.cz>
// Created on: Mon Aug  4 17:50:37 2003
//

#include <assert.h>
#include <stdlib.h>
#include "Global.h"

// zda muze aktualni hrac hrac s urcitym polickem
bool gsCanMove(const st_game_status *status,  int cellID)
{
	assert( status != NULL );
	
	// zda ma smysl testovat
	if ( cellID < 1 || cellID > status->cells ) return false; 
		
	// zkusime, zda neni 0 kamenu - pak polickem lze hrat
	return status->field[status->player][cellID] != 0;
}

// provede tah aktualnim hracem a zmeni hrace na druheho. Vraci false, pokud byl pokus o nedovoleny tah. Nekontroluje gsCanMove()
bool gsMakeMove(st_game_status *status, int cellID)
{
	assert( status != NULL );
	
	// zajistime ochranu pred "uzivatelem"
	if ( !gsCanMove(status, cellID) ) return false;
	
	// inicializace
	int p =  status->player; // ID radku, do ktereho budeme pridavat
	int c =  cellID-1;       // prihradka, kam se zrovna bude pridavat
	int r =  status->field[p][cellID]; // pocet kamenu, ktere musime rozhodat dale
	
	// zmena hraci plochy
	status->field[p][cellID] = 0; // policko, se kterym hybeme se vyprazdni
	while ( r > 1 ) {
		status->field[p][c]++;
		if ( c == 0 ) { // rozdavani kamenu musi "cyklit"
			c = status->cells;
			if ( p == 0 ) p = 1; // zmenime radku, do ktere se pridava
			else p = 0;
		}	else c--;
		r--;
	}
	
	// posledni kamen ma specialni pozici - viz. pravidla
	int p2;
	if ( p == 0 ) p2 = 1;
	else p2 = 0;
		
	if ( status->field[p][c] == 0 && c != 0 && status->field[p2][1+status->cells-c] != 0 ) {
		status->field[status->player][0] += 1 + status->field[p2][1+status->cells-c];
		status->field[p2][1+status->cells-c] = 0;
	}
	else status->field[p][c]++;
	
	// zmena aktualniho hrace, pokud posledni kamen nedopadl do domecku - viz. pravidla
	if ( c != 0 || p != status->player ) 
		if ( status->player == 0 ) status->player = 1;
		else status->player = 0;
			
	// konec zmeny stavu v tahu
	return true;
}

// zda muze aktualni hrac vubec hrat (ma nejaky mozny tah)
bool gsCanMove(st_game_status *status)
{
	assert( status != NULL );
	
	// prohlidneme vsechna policka a zjistime, zda je alespon nenulove
	int *j = status->field[status->player];
	j++;
	for (int i = 1; i <= status->cells; i++, j++) 
		if ( *j != 0 ) return true;
	
	return false;
	
}

// zda muze aktualni hrac vubec hrat (ma nejaky mozny tah)
bool gsCanMoveConst(const st_game_status *status)
{
	assert( status != NULL );
	
	int player = status->player;
	for (int i = 1; i <= status->cells; i++) 
		if ( status->field[player][i] != 0 ) return true;
			
	return false;
}

// provede tah aktualnim hracem a pripadne zmeni hrace na druheho. Vraci false, pokud byl pokuso nedovoleny tah.
bool gsMakeMoveCheck(st_game_status *status, int cellID)
{
	assert( status != NULL );
	
	// provedeme tah
	if ( !gsMakeMove(status, cellID) ) return false;
		
	// test na hru druheho hrace - zda vubec muze hrat
	if ( !gsCanMove(status) ) 
		if ( status->player == 1 ) status->player = 0;
		else status->player = 1;
			
	return true;
}

// vraci true, pokud hra zkoncila - kameny jsou jen v domecku
bool gsGameEnded(const st_game_status *status)
{
	for (int i = 1; i <= status->cells; i++) 
		if ( status->field[0][i] != 0 || status->field[1][i] != 0 ) return false;
			
	return true;
}
