//
// File: CAI.h
// Created by: berny <2berny@seznam.cz>
// Created on: Mon Aug  4 09:24:32 2003
//

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include "CAI.h"
#include "CGameControl.h"

// vlaknova funkce - aby pri premysleni program netuhnul
// predpoklada arg je ukazatel na CAI. Zpet vraci int cislo policka
// pretypovane na void *
void* CAI::AIthread(void *arg)
{
	assert( arg != NULL );
	
	// inicializace
	CAI* ai = (CAI*) arg;
	ai->m_thread_active = true;
	
	// nastaveni vlakna - povolime jeho okamzite zruseni
	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
	
	// vypocet
	int move = ai->GetCell();
	
	// konec
	ai->m_thread_active = false;
	return (void*) move;
}


CAI::CAI(CGameControl *gc)
{
	assert( gc != NULL );

	m_gc = gc;
	m_playing = false;
	
	m_thread_active = false; // vlakno samozrejme pasivni
}


CAI::~CAI()
{
	// kdyby nam tam nahodou zbylo nejake vlakno
	if ( m_thread_active ) {
		pthread_cancel(m_thread);
		m_thread_active = false;
	}
}

// z hlavni hry je potreba obcas udelat tick, aby se mohlo testovat ukonceni threatu, kdyby se nahodou pouzivaly
void CAI::TimerTick()
{
	// zapneme inteligenci random
	if ( !WantUserInput() && m_playing ) {
#ifdef _AWARY_MULTITHREAD_
			if ( m_thread_active == false ) { // kdyz vlakno dobehlo
				int move;
				pthread_join(m_thread, (void**) &move);
				m_gc->AIMoved( move, this);
				m_playing = false;
			}
#else		
		m_gc->AIMoved( GetCell(&m_status), this);
		m_playing = false;
#endif
	}
}	

// zda AI bude chtit vstup uzivatele
bool CAI::WantUserInput()
{
	return false;
}
void CAI::UserInput(int field)
{
	if ( WantUserInput() && m_playing ) {
		m_gc->AIMoved( field, this);
		m_playing = false;
	}
}

// volano z hlavni hry - predpoklada, ze vysledek se vrati..
void CAI::TryMove()
{
	m_status  = m_gc->GetStatus();
	m_playing = true;
	
#ifdef _AWARY_MULTITHREAD_
	// vytvorime vlakno, ve kterem nechame pocitac msylet a budeme se mezitim 
	// venovat uzivateli
	pthread_create(& m_thread, NULL, &AIthread, (void*) this);
#endif	
}

// vrati cellik, kam si inteligence mysli, ze by se melo hrat
// nepracuje s internim stavem, aby bylo mozne volat zvenku
// tohle je jen random, ktery ale nejprve overi, zda je mozne vubec
// nejaky tah provadet a zda je vyrandomovany tah povoleny
int CAI::GetCell(const st_game_status  *status)
{
	assert( status != NULL );
	
	if ( !gsCanMoveConst(status) ) return 0;
		
	int move = 1+rand()%status->cells;
	while ( !gsCanMove(status, move) ) 
		move = 1+rand()%status->cells;
	
	return move;
}

// jako predchozi, ale s vlastnim vnitrkem
int CAI::GetCell()
{
	return GetCell(&m_status);
}
