#include <gnome.h>
#include <assert.h>
#include <stdio.h>

#include "gui.h"

#include "CGameControl.h"
#include "CAI.h"
#include "CAIAB.h"
#include "CAIHuman.h"
#include "CAIEasy.h"
#include "tthread.h"

CGameControl *gc = NULL; // objekt hry

st_game_status test_stat; // zobrazovany status hry - pouziva se pri ruznych bonusech jako pri prejeti buttonku a tak podobne
bool test_stat_valid = false; // zda je platny
gboolean g_show_stars = 0;


CAI *ai1 = NULL; // inteligence do hry
CAI *ai2 = NULL; 

// kliknuti na domecek
void on_domecek_clicked (GtkButton *button, gpointer user_data)
{
	if ( gc != NULL ) {

		sem_wait(&g_sem);

		gc->TimerTick();
		gc->NextTurn();
		gc->TimerTick();

		if ( gc->StatusChanged() ) test_stat_valid = false;
 		sem_post(&g_sem);
		
		gui_actualize_game_status();
	}
	
	auto_turn();	
}

// prvni hrac chce hrat necim - (int)user_data je cislo policka, kterych chc hrat
void on_pl1_clicked (GtkButton *button, gpointer user_data)
{
	int data = (int)user_data;
	// posleme inteligenci, ze si uzivatel preje tahnout, pokud to chce
	if ( ai1 != NULL && ai1->WantUserInput() ) {
		
		sem_wait(&g_sem);
		
		gc->TimerTick();
		gc->NextTurn();
		ai1->UserInput(data);
		gc->TimerTick();
		
		test_stat_valid = false;
		gc->StatusChanged(); // jen si vybereme pripadnou zmenu
		sem_post(&g_sem);
		
		gui_actualize_game_status();
	}
	
	auto_turn();	
}

// druhy hrac chce hrat necim - (int)user_data je cislo policka, kterych chc hrat
void on_pl2_clicked (GtkButton *button, gpointer user_data)
{	
	int data = (int)user_data;
	// posleme inteligenci, ze si uzivatel preje tahnout, pokud to chce
	if ( ai2 != NULL && ai2->WantUserInput() ) {

		sem_wait(&g_sem);

		gc->TimerTick();
		gc->NextTurn();
		ai2->UserInput(data);
		gc->TimerTick();		

		test_stat_valid = false;
		gc->StatusChanged();
		sem_post(&g_sem);

		gui_actualize_game_status();
	}	
	
	auto_turn();	
}

// aktualizuje napisy na knoflikach
void gui_actualize_game_status()
{
	sem_wait(&g_sem);
	
	if ( gc != NULL /* && gc->StatusChanged() */) {
		st_game_status gs;
		if ( test_stat_valid	) gs = test_stat;
		else gs = gc->GetStatus();

		char s[200];
		GtkWidget *gw = NULL;
		int ai[2];
		ai[0] = g_ai1;	ai[1] = g_ai2;	

	// aktualizujeme knofliky na hraci plose
		for ( int i = 0; i < 2; i++ ) {
			for ( int j = 0; j <= gs.cells; j++) {
				// najdeme widget
				sprintf(s, "bpl%d_%d", i+1, j);
				gw = lookup_widget(main_window, s);
				if ( gw != NULL ) {
					// zmenime nadpis
					s[0] = '\0';
					sprintf(s, "%d", gs.field[i][j]);
					
					if ( g_show_stars ) { // pokus si uzivatel preje, nakreslime mu hvezdicky
						int l = strlen(s);
						for ( int k = 0; k < gs.field[i][j] && k < 190; k++, l++ ) {
							if ( k % 5 == 0 ) s[l++] = '\n';
							s[l] = '*';
						}
						s[l] = '\0';
					}
					
					gtk_button_set_label(GTK_BUTTON(gw), s);
					// nastavime, zda je moznost na nej kliknout
					if ( ( j == 0 || ( gs.player == i && ai[i] == AI_HUMAN ) )
 						  && gc->GameEnded() == GAME_ENDED_NO)
					  	gtk_widget_set_sensitive (gw, TRUE);
					else
					  	gtk_widget_set_sensitive (gw, FALSE);
					
				}
			}
		}
	
		// aktualizace statusbaru
		gw = lookup_widget(main_window, "statusbar");
		s[0] = '\0';
		switch ( gc->GameEnded() ) {
		case GAME_ENDED_NO :
			sprintf(s, "%d. tah, hraje hrac %d. ", gc->GetTurnsCnt()+1, gs.player + 1);
			break;
		case GAME_ENDED_1ST :
			sprintf(s, "Prvni hrac vyhral.");
			break;
		case GAME_ENDED_2ND :
			sprintf(s, "Druhy hrac vyhral");
			break;
		case GAME_ENDED :
			sprintf(s, "Hra zkoncila remizou");
			break;
		}
		gtk_statusbar_push(GTK_STATUSBAR(gw), 0, s);
	}
	sem_post(&g_sem);
}

// tvori AI podle aitype - je podle #define z gui.h
CAI* create_AI(int aitype, CGameControl *gc)
{
	switch (aitype){ // nemusi byt breaky
	case AI_HUMAN :
		return new CAIHuman(gc);
	case AI_RANDOM :
		return new CAI(gc);
	case AI_EASY :
		return new CAIEasy(gc);
	case AI_AB4 :
		return new CAIAB(gc, 4);
	case AI_AB8 :
		return new CAIAB(gc, 8);
	case AI_AB12 :
		return new CAIAB(gc, 12);
		
	}
	
	fprintf(stderr, "Neznamy typ AI : %d !! \n", aitype);
	return NULL; // nejaka blbost	
}

void start_new_game()
{
	printf("Zacinam novou hru\n AI1 : %d, AI2 : %d, Cells : %d, Rocks : %d\n",  g_ai1, g_ai2, g_cells, g_rocks);

	gui_recreate_gamedesk(g_cells);

	
	// start nove hry 	
	clean_game_data(); // zrusime starou hru
	
	sem_wait(&g_sem);
	gc = new CGameControl();
	ai1 = create_AI(g_ai1, gc);
	ai2 = create_AI(g_ai2, gc);
	gc->NewGame(g_cells, g_rocks, ai1, ai2, false);
	test_stat_valid = false;
	sem_post(&g_sem);
	
	// zobrazime startovni pozici - aktualizace
	gui_actualize_game_status();
	auto_turn();
}

// funkce, ktera by se mela volat na konci programu - provadi uklid
void clean_game_data()
{
	sem_wait(&g_sem);
	
	if ( gc != NULL ) delete gc;
	gc = NULL;
	
	if ( ai1 != NULL ) delete ai1;
	ai1 = NULL;
	
	if ( ai2 != NULL ) delete ai2;
	ai2 = NULL;
	
	sem_post(&g_sem);
}

// pri prejeti myskou - pro delani "ukazky" noveho stavu
void on_pl_enter (GtkButton *button, gpointer user_data)
{
	
	
	if ( gc != NULL ) {
		sem_wait(&g_sem);
		test_stat = gc->GetStatus();
		
		int player = test_stat.player;
		gsMakeMove(&test_stat, (int) user_data);
		test_stat.player = player;
	
		test_stat_valid = true;
		sem_post(&g_sem);	
		
		gui_actualize_game_status();
	}
	

}

void on_pl_leave (GtkButton *button, gpointer user_data)
{
	sem_wait(&g_sem);
	test_stat_valid = false;
	sem_post(&g_sem);	
	
	gui_actualize_game_status();
}
